@@ -107,6 +107,8 @@ struct _GdkToplevelX11
   guint have_maxhorz : 1;       /* _NET_WM_STATE_MAXIMIZED_HORZ */
   guint have_fullscreen : 1;    /* _NET_WM_STATE_FULLSCREEN */
 
+  guint is_leader : 1;
+  
   gulong map_serial;	/* Serial of last transition from unmapped */
   
   GdkPixmap *icon_pixmap;
@@ -114,6 +116,9 @@ struct _GdkToplevelX11
   GdkPixmap *icon_window;
   GdkWindow *group_leader;
 
+  /* Time of most recent user interaction. */
+  gulong user_time;
+
   /* We use an extra X window for toplevel windows that we XSetInputFocus()
    * to in order to avoid getting keyboard events redirected to subwindows
    * that might not even be part of this app
