@@ -25,6 +25,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include "gdkconfig.h"
 #include <math.h>
 
@@ -57,6 +58,7 @@
 #include "gtkintl.h"
 #include "gtkimage.h"
 #include "gtkstock.h"
+#include "gtkaccessible.h"
 
 #include <string.h>
 
@@ -169,6 +171,10 @@ static void     default_noscreen_change_palette_func    (const GdkColor    *colo
 static void     default_change_palette_func             (GdkScreen	   *screen,
 							 const GdkColor    *colors,
 							 gint               n_colors);
+static void     make_control_relations                  (AtkObject         *atk_obj,
+                                                         GtkWidget         *widget);
+static void     make_all_relations                      (AtkObject         *atk_obj,
+                                                         ColorSelectionPrivate *priv);
 
 static gpointer parent_class = NULL;
 static guint color_selection_signals[LAST_SIGNAL] = { 0 };
@@ -1758,29 +1764,29 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_OPACITY_CONTROL,
                                    g_param_spec_boolean ("has_opacity_control",
-							 _("Has Opacity Control"),
-							 _("Whether the color selector should allow setting opacity"),
+							 P_("Has Opacity Control"),
+							 P_("Whether the color selector should allow setting opacity"),
 							 FALSE,
 							 G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_PALETTE,
                                    g_param_spec_boolean ("has_palette",
-							 _("Has palette"),
-							 _("Whether a palette should be used"),
+							 P_("Has palette"),
+							 P_("Whether a palette should be used"),
 							 FALSE,
 							 G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_CURRENT_COLOR,
                                    g_param_spec_boxed ("current_color",
-                                                       _("Current Color"),
-                                                       _("The current color"),
+                                                       P_("Current Color"),
+                                                       P_("The current color"),
                                                        GDK_TYPE_COLOR,
                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_CURRENT_ALPHA,
                                    g_param_spec_uint ("current_alpha",
-						      _("Current Alpha"),
-						      _("The current opacity value (0 fully transparent, 65535 fully opaque)"),
+						      P_("Current Alpha"),
+						      P_("The current opacity value (0 fully transparent, 65535 fully opaque)"),
 						      0, 65535, 65535,
 						      G_PARAM_READABLE | G_PARAM_WRITABLE));
   
@@ -1794,8 +1800,8 @@ gtk_color_selection_class_init (GtkColorSelectionClass *klass)
 		  G_TYPE_NONE, 0);
 
   gtk_settings_install_property (g_param_spec_string ("gtk-color-palette",
-                                                      _("Custom palette"),
-                                                      _("Palette to use in the color selector"),
+                                                      P_("Custom palette"),
+                                                      P_("Palette to use in the color selector"),
                                                       default_colors,
                                                       G_PARAM_READWRITE));
 }
@@ -1812,6 +1818,7 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
   GtkWidget *picker_image;
   gint i, j;
   ColorSelectionPrivate *priv;
+  AtkObject *atk_obj;
   
   gtk_widget_push_composite_child ();
 
@@ -1928,7 +1935,7 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
                         priv->hex_entry,
                         _("You can enter an HTML-style hexadecimal color value, or simply a color name such as 'orange' in this entry."), NULL);
   
-  gtk_widget_set_size_request (priv->hex_entry, 75, -1);  
+  gtk_entry_set_width_chars (GTK_ENTRY (priv->hex_entry), 7);
   gtk_table_set_col_spacing (GTK_TABLE (table), 3, 15);
   gtk_table_attach_defaults (GTK_TABLE (table), priv->hex_entry, 1, 5, 5, 6);
   
@@ -1974,6 +1981,14 @@ gtk_color_selection_init (GtkColorSelection *colorsel)
       gtk_widget_hide (priv->palette_frame);
     }
 
+  atk_obj = gtk_widget_get_accessible (priv->triangle_colorsel);
+  if (GTK_IS_ACCESSIBLE (atk_obj))
+    {
+      atk_object_set_name (atk_obj, _("Color Wheel"));
+      atk_object_set_role (gtk_widget_get_accessible (GTK_WIDGET (colorsel)), ATK_ROLE_COLOR_CHOOSER);
+      make_all_relations (atk_obj, priv);
+    } 
+
   gtk_widget_pop_composite_child ();
 }
 
@@ -2209,13 +2224,14 @@ gtk_color_selection_set_has_palette (GtkColorSelection *colorsel,
  **/
 void
 gtk_color_selection_set_current_color (GtkColorSelection *colorsel,
-				       GdkColor          *color)
+				       const GdkColor    *color)
 {
   ColorSelectionPrivate *priv;
   gint i;
   gboolean previous_changed;
   
   g_return_if_fail (GTK_IS_COLOR_SELECTION (colorsel));
+  g_return_if_fail (color != NULL);
 
   priv = colorsel->private_data;
   priv->changing = TRUE;
@@ -2301,6 +2317,7 @@ gtk_color_selection_get_current_color (GtkColorSelection *colorsel,
   ColorSelectionPrivate *priv;
   
   g_return_if_fail (GTK_IS_COLOR_SELECTION (colorsel));
+  g_return_if_fail (color != NULL);
   
   priv = colorsel->private_data;
   color->red = UNSCALE (priv->color[COLORSEL_RED]);
@@ -2363,11 +2380,12 @@ gtk_color_selection_get_color (GtkColorSelection *colorsel,
  **/
 void
 gtk_color_selection_set_previous_color (GtkColorSelection *colorsel,
-					GdkColor          *color)
+					const GdkColor    *color)
 {
   ColorSelectionPrivate *priv;
   
   g_return_if_fail (GTK_IS_COLOR_SELECTION (colorsel));
+  g_return_if_fail (color != NULL);
   
   priv = colorsel->private_data;
   priv->changing = TRUE;
@@ -2424,6 +2442,7 @@ gtk_color_selection_get_previous_color (GtkColorSelection *colorsel,
   ColorSelectionPrivate *priv;
   
   g_return_if_fail (GTK_IS_COLOR_SELECTION (colorsel));
+  g_return_if_fail (color != NULL);
   
   priv = colorsel->private_data;
   color->red = UNSCALE (priv->old_color[COLORSEL_RED]);
@@ -2750,3 +2769,26 @@ gtk_color_selection_set_change_palette_with_screen_hook (GtkColorSelectionChange
 
   return old;
 }
+
+static void
+make_control_relations (AtkObject *atk_obj,
+                        GtkWidget *widget)
+{
+  AtkObject *obj;
+
+  obj = gtk_widget_get_accessible (widget);
+  atk_object_add_relationship (atk_obj, ATK_RELATION_CONTROLLED_BY, obj);
+  atk_object_add_relationship (obj, ATK_RELATION_CONTROLLER_FOR, atk_obj);
+}
+
+static void
+make_all_relations (AtkObject *atk_obj,
+                    ColorSelectionPrivate *priv)
+{
+  make_control_relations (atk_obj, priv->hue_spinbutton);
+  make_control_relations (atk_obj, priv->sat_spinbutton);
+  make_control_relations (atk_obj, priv->val_spinbutton);
+  make_control_relations (atk_obj, priv->red_spinbutton);
+  make_control_relations (atk_obj, priv->green_spinbutton);
+  make_control_relations (atk_obj, priv->blue_spinbutton);
+}
