@@ -58,6 +58,8 @@ struct _GtkFileChooserEntry
 
   guint has_completion : 1;
   guint in_change      : 1;
+
+  guint sorting_idle;
 };
 
 enum
@@ -205,6 +207,9 @@ gtk_file_chooser_entry_finalize (GObject *object)
 {
   GtkFileChooserEntry *chooser_entry = GTK_FILE_CHOOSER_ENTRY (object);
 
+  if (chooser_entry->sorting_idle)
+    g_source_remove (chooser_entry->sorting_idle);
+
   if (chooser_entry->completion_store)
     g_object_unref (chooser_entry->completion_store);
 
@@ -513,6 +518,17 @@ add_completion_idle (GtkFileChooserEntry *chooser_entry)
     }
 }
 
+static gboolean
+sorting_idle (GtkFileChooserEntry *chooser_entry)
+{
+  chooser_entry->sorting_idle = 0;
+
+  /* Turn sorting back on */
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (chooser_entry->completion_store),
+					DISPLAY_NAME_COLUMN, GTK_SORT_ASCENDING);
+
+  return FALSE;
+}
 
 static void
 update_current_folder_files (GtkFileChooserEntry *chooser_entry,
@@ -522,7 +538,14 @@ update_current_folder_files (GtkFileChooserEntry *chooser_entry,
 
   g_assert (chooser_entry->completion_store != NULL);
 
-  /* Bah.  Need to turn off sorting */
+  if (chooser_entry->sorting_idle == 0)
+    {
+      /* Turn off sorting, since it is slow with the default list store implementation */
+      gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (chooser_entry->completion_store),
+					    -2, GTK_SORT_ASCENDING);
+      chooser_entry->sorting_idle = g_idle_add (sorting_idle, chooser_entry);
+    }
+
   for (tmp_list = added_uris; tmp_list; tmp_list = tmp_list->next)
     {
       GtkFileInfo *info;
@@ -535,23 +558,20 @@ update_current_folder_files (GtkFileChooserEntry *chooser_entry,
 				       NULL); /* NULL-GError */
       if (info)
 	{
-	  const gchar *display_name = gtk_file_info_get_display_name (info);
+	  const gchar *display_name;
 	  GtkTreeIter iter;
 
+	  display_name = gtk_file_info_get_display_name (info);
+
 	  gtk_list_store_append (chooser_entry->completion_store, &iter);
 	  gtk_list_store_set (chooser_entry->completion_store, &iter,
 			      DISPLAY_NAME_COLUMN, display_name,
 			      PATH_COLUMN, path,
 			      -1);
-
 	  gtk_file_info_free (info);
 	}
     }
 
-  /* FIXME: we want to turn off sorting temporarily.  I suck... */
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (chooser_entry->completion_store),
-					DISPLAY_NAME_COLUMN, GTK_SORT_ASCENDING);
-
   add_completion_idle (chooser_entry);
 }
 
@@ -660,11 +680,16 @@ gtk_file_chooser_entry_focus (GtkWidget        *widget,
       (GTK_WIDGET_HAS_FOCUS (widget)) &&
       (! control_pressed))
     {
+      gint pos = 0;
+
       if (chooser_entry->has_completion)
-	{
-	  gtk_editable_set_position (GTK_EDITABLE (widget),
-				     GTK_ENTRY (widget)->text_length);
-	}
+	gtk_editable_set_position (GTK_EDITABLE (widget),
+				   GTK_ENTRY (widget)->text_length);
+      /* Trigger the completion window to pop up again by a 
+       * zero-length insertion, a bit of a hack.
+       */
+      gtk_editable_insert_text (GTK_EDITABLE (widget), "", -1, &pos);
+
       return TRUE;
     }
   else
