@@ -695,6 +695,14 @@ get_default_icons (GtkIconFactory *factory)
 	     16, stock_up_arrow_16,
 	     24, stock_up_arrow_24);
 
+  add_icon2 (factory, "gtk-file",
+	     16, stock_file_16,
+	     24, stock_file_24);
+
+  add_icon2 (factory, "gtk-directory",
+	     16, stock_directory_16,
+	     24, stock_directory_24);
+
   /* Generic size only */
 
   add_icon (factory, GTK_STOCK_CLEAR, 24, stock_clear_24);
@@ -1595,7 +1603,7 @@ render_icon_name_pixbuf (GtkIconSource    *icon_source,
   
   if (widget && gtk_widget_has_screen (widget))
     screen = gtk_widget_get_screen (widget);
-  else if (style->colormap)
+  else if (style && style->colormap)
     screen = gdk_colormap_get_screen (style->colormap);
   else
     {
@@ -1768,7 +1776,7 @@ gtk_icon_set_render_icon (GtkIconSet        *icon_set,
   GdkPixbuf *icon;
   
   g_return_val_if_fail (icon_set != NULL, NULL);
-  g_return_val_if_fail (GTK_IS_STYLE (style), NULL);
+  g_return_val_if_fail (style == NULL || GTK_IS_STYLE (style), NULL);
 
   if (icon_set->sources == NULL)
     return render_fallback_image (style, direction, state, size, widget, detail);
@@ -2656,10 +2664,14 @@ static void
 clear_cache (GtkIconSet *icon_set,
              gboolean    style_detach)
 {
-  GSList *tmp_list;
+  GSList *cache, *tmp_list;
   GtkStyle *last_style = NULL;
 
-  tmp_list = icon_set->cache;
+  cache = icon_set->cache;
+  icon_set->cache = NULL;
+  icon_set->cache_size = 0;
+  
+  tmp_list = cache;
   while (tmp_list != NULL)
     {
       CachedIcon *icon = tmp_list->data;
@@ -2683,9 +2695,7 @@ clear_cache (GtkIconSet *icon_set,
       tmp_list = g_slist_next (tmp_list);
     }
 
-  g_slist_free (icon_set->cache);
-  icon_set->cache = NULL;
-  icon_set->cache_size = 0;
+  g_slist_free (cache);
 }
 
 static GSList*
