@@ -24,6 +24,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+#include <config.h>
 #include <gdk/gdk.h>
 #include "gtkinvisible.h"
 #include "gtkintl.h"
@@ -111,8 +112,8 @@ gtk_invisible_class_init (GtkInvisibleClass *class)
   g_object_class_install_property (gobject_class,
 				   PROP_SCREEN,
 				   g_param_spec_object ("screen",
- 							_("Screen"),
- 							_("The screen where this window will be displayed"),
+ 							P_("Screen"),
+ 							P_("The screen where this window will be displayed"),
 							GDK_TYPE_SCREEN,
  							G_PARAM_READWRITE));
 }
@@ -146,6 +147,8 @@ gtk_invisible_destroy (GtkObject *object)
       invisible->has_user_ref_count = FALSE;
       g_object_unref (invisible);
     }
+
+  GTK_OBJECT_CLASS (parent_class)->destroy (object);  
 }
 
 /**
