@@ -26,6 +26,7 @@
 
 #define GTK_MENU_INTERNALS
 
+#include <config.h>
 #include <string.h>
 #include "gtkaccellabel.h"
 #include "gtkmain.h"
@@ -501,7 +502,7 @@ gtk_menu_item_size_request (GtkWidget      *widget,
   g_return_if_fail (requisition != NULL);
 
   gtk_widget_style_get (widget,
- 			"horizontal padding", &horizontal_padding,
+ 			"horizontal_padding", &horizontal_padding,
 			NULL);
   
   bin = GTK_BIN (widget);
@@ -961,7 +962,8 @@ gtk_menu_item_select_timeout (gpointer data)
 
   parent = GTK_WIDGET (menu_item)->parent;
 
-  if (parent && GTK_IS_MENU_SHELL (parent) && GTK_MENU_SHELL (parent)->active)
+  if ((GTK_IS_MENU_SHELL (parent) && GTK_MENU_SHELL (parent)->active) || 
+      (GTK_IS_MENU (parent) && GTK_MENU (parent)->torn_off))
     {
       gtk_menu_item_popup_submenu (data);
       if (menu_item->timer_from_keypress && menu_item->submenu)
