@@ -285,7 +285,7 @@ gtk_paned_class_init (GtkPanedClass *class)
    * Since: 2.4
    */
   g_object_class_install_property (object_class,
-				   PROP_MIN_POSITION,
+				   PROP_MAX_POSITION,
 				   g_param_spec_int ("max_position",
 						     P_("Maximal Position"),
 						     P_("Largest possible value for the \"position\" property"),
@@ -326,7 +326,7 @@ gtk_paned_class_init (GtkPanedClass *class)
 								    TRUE,
 								    G_PARAM_READWRITE));
 
-  signals [CYCLE_HANDLE_FOCUS] =
+  signals [CYCLE_CHILD_FOCUS] =
     g_signal_new ("cycle_child_focus",
 		  G_TYPE_FROM_CLASS (object_class),
 		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
@@ -1192,9 +1192,9 @@ gtk_paned_compute_position (GtkPaned *paned,
       else if (!paned->child1_resize && paned->child2_resize)
 	paned->child1_size = child1_req;
       else if (child1_req + child2_req != 0)
-	paned->child1_size = allocation * ((gdouble)child1_req / (child1_req + child2_req));
+	paned->child1_size = allocation * ((gdouble)child1_req / (child1_req + child2_req)) + 0.5;
       else
-	paned->child1_size = allocation * 0.5;
+	paned->child1_size = allocation * 0.5 + 0.5;
     }
   else
     {
@@ -1206,7 +1206,7 @@ gtk_paned_compute_position (GtkPaned *paned,
 	  if (paned->child1_resize && !paned->child2_resize)
 	    paned->child1_size += allocation - paned->last_allocation;
 	  else if (!(!paned->child1_resize && paned->child2_resize))
-	    paned->child1_size = allocation * ((gdouble) paned->child1_size / (paned->last_allocation));
+	    paned->child1_size = allocation * ((gdouble) paned->child1_size / (paned->last_allocation)) + 0.5;
 	}
     }
 
